## Copyright (C) 2005 by Dr. Dieter Maurer <dieter@handshake.de>
# D-66386 St. Ingbert, Eichendorffstr. 23, Germany
# See "LICENSE.txt" in the top package directory, for details.
#       $Id: forwarder.py,v 1.1.1.1 2005/03/07 19:24:38 dieter Exp $
'''LocalEventForwarder.

See '__init__.py' for a description.
'''

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from BTrees.OOBTree import OOBTree
from OFS.SimpleItem import SimpleItem

from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from zope.interface import implements
from zope.app.event.interfaces import IObjectEvent

from dm.z23 import adapters

from interfaces import ILocalEventForwarder


class Forwarder(SimpleItem):
  meta_type = 'Local Event Forwarder'
  id = 'LocalEventForwarder'
  title = 'Local Event Forwarder'

  implements(ILocalEventForwarder)

  security = ClassSecurityInfo()
  security.declareProtected('View management screens',
                            'subscribe', 'unsubscribe', 'listSubscriptions'
                            )

  def __init__(self):
    self._subs = OOBTree()
    self._ticket = 0

  def subscribe(self, path, notifier):
    '''see interface.'''
    if isinstance(path, str): path = path.split('/')
    path = tuple(path)
    if not path[0]: path = path[1:]
    ticket = self._getTicket()
    subs = self._subs.get(path)
    if subs is None: subs = []
    subs.append((ticket, notifier))
    self._subs[path] = subs
    return ticket

  def unsubscribe(self, tickets):
    '''see interface.'''
    if isinstance(tickets, str): tickets = (tickets,)
    for subs in list(self._subs.items()):
      path, subs = subs; modified = False; i = 0
      while i < len(subs):
        if subs[i][0] in tickets:
          modified = True; del subs[i]
        else: i += 1
      if modified:
        if subs: self._subs[path] = subs
        else: del self._subs[path]

  def listSubscriptions(self):
    '''see interface.'''
    l = []
    for subs in self._subs.items():
      path, subs = subs
      for ticket, notifier in subs:
        l.append((ticket, path, notifier))
    return l
    
  def _getTicket(self):
    ticket = `self._ticket`
    self._ticket += 1
    return ticket

  def _notify(self, object, event):
    # this is efficient only, when the number of subcriptions is small
    # (what we expect). Otherwise, a more efficient data structure
    # could be used (linear in the path length).
    path = object.getPhysicalPath()[1:]; pl = len(path)
    subs = self._subs
    for subpath in subs.keys(None,path):
      spl = len(subpath)
      if spl <= pl and subpath == path[:spl]:
        for sub in subs[subpath]: sub[1](object, event)





def _notifier(event):
  object = event.object
  forwarder = object.getPhysicalRoot().Control_Panel.LocalEventForwarder
  forwarder._notify(object, event)


def initialize(context):
  control_panel = context._ProductContext__app.Control_Panel
  if not hasattr(control_panel.aq_base, Forwarder.id):
    forwarder = Forwarder()
    control_panel._setObject(forwarder.getId(), forwarder)
  adapters.subscribe((IObjectEvent,), None, _notifier)
