## Copyright (C) 2005 by Dr. Dieter Maurer <dieter@handshake.de>
# D-66386 St. Ingbert, Eichendorffstr. 23, Germany
# See "LICENSE.txt" in the top package directory, for details.
#       $Id: interfaces.py,v 1.1.1.1 2005/03/07 19:24:38 dieter Exp $

from zope.interface import Interface

class ILocalEventForwarder(Interface):
  '''a global service forwarding local events.'''

  def subscribe(path, notifier):
    '''subscribe for *notifier* calls for events affecting objects at or below *path*.

    *notifier* will be called with object and event as arguments.

    Return a cancelation ticket to undo the subscription.

    Note, that *notifier* must be picklable.
    The 'IAqWrapperLess' interface might help you meet this restriction.
    '''

  def unsubscribe(tickets):
    '''remove the subscription identified by *tickets*.'''

  def listSubscriptions():
    '''the list of subscriptions, each identified by a triple
    *cancelation ticket*, *path* and *notifier*.
    '''


class ILocalEventSubscription(Interface):
  '''a local service for local event subscriptions.

  Note that we cannot provide 'unsubscribe' because the underlaying
  Zope3 object does not support it.
  But, if necessary, we can work around this...
  '''

  def subscribe(required, notifier):
    '''call *notifier* for events satisfying *required*.

    Note, that the forwarder (i.e. the primary event source)
    forwards events of the form *object, event*.
    Note that 'notifier' must have an object in the Zope2 site hierarchy
    as first argument, if 'IaqWrapperLess' is used in its construction.
    Especially, this means that it cannot be an event.

    Note, that *notifier* must be picklable.
    The interfaces 'IAqWrapperLess' and 'IPicklableMethods'
    from 'dm.z23.interfaces' might help you meet this restriction.
    '''

  def listSubscriptions():
    '''the list of subscriptions, each identified by a pair
    *required* and *notifier*.
    '''

  def notify(*objects):
    '''notify an event identified by *objects*.'''
