## Copyright (C) 2005 by Dr. Dieter Maurer <dieter@handshake.de>
# D-66386 St. Ingbert, Eichendorffstr. 23, Germany
# See "LICENSE.txt" in the top package directory, for details.
#       $Id: subscription.py,v 1.1.1.1 2005/03/07 19:24:38 dieter Exp $
'''Local event subscription utility.

This utility is intended as a CMF tool which statically interfaces
between various other CMF tools.

It is not intended for short lived subscriptions nor for
subscriptions set up via management screens.
It therefore lacks management pages.
However, you can use its 'listSubsciptions' method to provide
an informative view for the tool.

It requires the 'LocalEventForwarder' service to be installed.
'''

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.CMFCorePermissions import ManagePortal


from zope.interface import implements

from dm.z23.adapter import PureLocalAdapterRegistry
from dm.z23.interfaces import IAqWrapperLess

from interfaces import ILocalEventSubscription

_mdoc = __doc__

class Subscription(UniqueObject, PureLocalAdapterRegistry, SimpleItem):
  __doc__ = _mdoc

  id = 'local_event_subscription'

  implements(ILocalEventSubscription)

  security = ClassSecurityInfo()
  security.declareProtected(ManagePortal,
                            'listSubscriptions',
                            )
  security.declarePrivate('subscribe', 'notify',
                          'manage_afterAdd', 'manage_beforeDelete',
                          )

  _registry_subscribe = PureLocalAdapterRegistry.subscribe

  def subscribe(self, required, notifier):
    '''see interface.'''
    self._registry_subscribe(required, None, notifier)

  def listSubscriptions(self):
    '''see interface.'''
    l = []
    for subs in self._subs.values():
      for sub in subs[-1]: l.append((subs[0], sub))
    return l

  def notify(self, *objects):
    return self.subscribers(objects, None)

  __forwarder_ticket = None
  def manage_afterAdd(self, item, container):
    forwarder = self.getPhysicalRoot().Control_Panel.LocalEventForwarder
    self.__forwarder_ticket = forwarder.subscribe(
      self.getPhysicalPath()[:-1],
      IAqWrapperLess(self).notify
      )

  def manage_beforeDelete(self, item, container):
    forwarder = self.getPhysicalRoot().Control_Panel.LocalEventForwarder
    forwarder.unsubscribe(self.__forwarder_ticket)

InitializeClass(Subscription)


def initialize(context):
  # empty, as we do not expect this tool to be instantiated via
  # the management interface
  pass



