## Copyright (C) 2005 by Dr. Dieter Maurer <dieter@handshake.de>
# D-66386 St. Ingbert, Eichendorffstr. 23, Germany
# See "LICENSE.txt" in the top package directory, for details.
#       $Id: test_localevent.py,v 1.1.1.1 2005/03/07 19:24:38 dieter Exp $
''''localevent' tests.'''

# as specified by "ZopeTestCase.framework"
import os, sys
if __name__ == '__main__':
  execfile(os.path.join(sys.path[0], 'framework.py'))
else: import framework

from unittest import TestSuite, makeSuite

from Testing.ZopeTestCase import ZopeTestCase, installProduct

from OFS.SimpleItem import SimpleItem

from zope.interface.verify import verifyObject
from zope.app.event.objectevent import modified
from zope.app.event.interfaces import IObjectEvent

from dm.z23.interfaces import IAqWrapperLess, IPicklableMethods

from Products.localevent.interfaces import ILocalEventForwarder, ILocalEventSubscription
from Products.localevent.subscription import Subscription

installProduct('localevent', 1)

class TestLocalEvent(ZopeTestCase):
  def afterSetUp(self):
    folder = self.folder
    folder.manage_addFolder('portal')
    portal = self.portal = folder.portal
    les = Subscription(); id = les.getId()
    portal._setObject(id, les)
    self.les = getattr(portal, id)
    folder.manage_addFile('obj1')
    portal.manage_addFile('obj2')

  # apparently, 'verifyObject' is broken! Deactivate test
  def _testInterfaces(self):
    verifyObject(
      ILocalEventForwarder,
      self.folder.Control_Panel.LocalEventForwarder,
                 )
    verifyObject(ILocalEventSubscription, self.les)

  def testNonPersistentNotification(self):
    self._testNotification(False)

  def testPersistentNotification(self):
    self._testNotification(True)

  def testProperDelete(self):
    forwarder = self.folder.Control_Panel.LocalEventForwarder
    self.assertEqual(len(forwarder.listSubscriptions()), 1)
    self.portal.manage_delObjects('local_event_subscription')
    self.assertEqual(len(forwarder.listSubscriptions()), 0)

  def _testNotification(self, persistent):
    subscriber = self._subscribe(persistent)
    # check 'listSubscription' (a little abuseive)
    sl = self.les.listSubscriptions()
    self.assertEqual(len(sl), 1)
    self.assertEqual(len(sl[0]), 2)
    self.assertEqual(sl[0][0], (None, IObjectEvent))
    # check notification
    modified(self.folder.obj1)
    modified(self.portal.obj2)
    self.assertEqual(subscriber.notifications, ['obj2'])

  def _subscribe(self, persistent):
    if persistent:
      from Products.localevent.tests.test_localevent import _PersistentSubscriber
      subscriber = _PersistentSubscriber(); id = subscriber.getId()
      self.portal._setObject(id, subscriber)
      subscriber = getattr(self.portal, id)
      notify = IAqWrapperLess(subscriber).notify
    else: 
      from Products.localevent.tests.test_localevent import _Subscriber
      subscriber = _Subscriber()
      notify = IPicklableMethods(subscriber).notify
    self.les.subscribe((None, IObjectEvent), notify)
    # to detect pickling problems
    get_transaction().commit(1)
    return subscriber

class _Subscriber:
  id = 'subscriber'
  def __init__(self):
    self.notifications = []

  def notify(self, object, event):
    self.notifications.append(object.getId())

  def getId(self): return self.id

class _PersistentSubscriber(_Subscriber, SimpleItem):
  pass
    

def test_suite():
  return TestSuite([makeSuite(TestLocalEvent, 'test')])

if __name__ == '__main__': framework()

